// u_c_person
// 2013-02-09

// -- (C) John COLIBRI 2013
// -- documentation: http://www.jcolibri.com

(*$r+*)

unit u_display_rtti;
  interface
    uses Classes, TypInfo, Rtti;

    procedure fill_field_list(p_c_object: tObject; p_c_strings: tStrings);
    procedure fill_method_list(p_c_object: tObject; p_c_strings: tStrings);
    procedure fill_property_list(p_c_object: tObject; p_c_strings: tStrings);

    function f_property_string_value(p_c_object: tObject; p_property_display: String): String;
    procedure set_property_value(p_c_object: tObject; p_property_display, p_string_value: String);

    procedure fill_ancestor_list(p_c_object: tObject; p_c_strings: tStrings);

    function f_display_type_kind(p_type_kind: tTypeKind): String;

    var g_c_unknown_type_kind: tStringList= Nil;

  implementation
    uses SysUtils, u_display_simple
        //, u_display_hex_2_simple
        ;

    const k_basic_type_kinds=
              [tkInteger, tkFloat
               , tkChar, tkString, tkWChar, tkLString, tkWString, tkUString
               , tkEnumeration
               ];

    function f_display_type_kind(p_type_kind: tTypeKind): String;
      begin
        Result:= GetEnumName(TypeInfo(tTypeKind), Ord(p_type_kind));
      end; // f_display_type_kind

    procedure check_unknown_type_kind_creation;
      begin
        if g_c_unknown_type_kind= Nil
          then begin
              g_c_unknown_type_kind:= tStringList.Create;
              g_c_unknown_type_kind.Sorted:= True;
            end;
      end; // check_unknown_type_kind_creation

    procedure fill_field_list(p_c_object: tObject; p_c_strings: tStrings);
      Var l_rtti_context: tRttiContext;
          l_c_rtti_type: tRttiType;
          l_c_rtti_field: tRttifield;
          l_field_name, l_field_type_name, l_type_kind_name, l_field_value: String;
      Begin
        check_unknown_type_kind_creation;

        p_c_strings.Clear;

        l_c_rtti_type:= l_rtti_context.GetType(p_c_object.ClassType);
        For l_c_rtti_field In l_c_rtti_type.GetFields Do
        Begin
          l_field_name:= l_c_rtti_field.Name;
          l_field_type_name:= l_c_rtti_field.FieldType.Name;
          l_type_kind_name:= f_display_type_kind(l_c_rtti_field.fieldType.TypeKind);

          If l_c_rtti_field.fieldType.TypeKind In
              [tkInteger, tkFloat
               , tkChar, tkString, tkWChar, tkLString, tkWString, tkUString
               , tkEnumeration
               ]
            Then l_field_value:=
                l_c_rtti_field.GetValue(p_c_object).ToString
            Else begin
                l_field_value:= '?' ;
                g_c_unknown_type_kind.Add(l_type_kind_name)
              end;

//          p_c_strings.Add(Format('%-25s %-10s %-15s %s',
          p_c_strings.Add(Format('%-19s %-18s %-15s %s',
              [l_field_name, l_field_type_name, l_type_kind_name, l_field_value]));
        End; // l_c_rtti_field
      End; // fill_field_list

    procedure fill_property_list(p_c_object: tObject; p_c_strings: tStrings);
      Var l_rtti_context: tRttiContext;
          l_c_rtti_type: tRttiType;
          l_c_rtti_property: tRttiProperty;
          l_property_name, l_property_type_name, l_type_kind_name, l_property_value: String;
          l_display: String;
      Begin
        check_unknown_type_kind_creation;

        p_c_strings.Clear;

        l_c_rtti_type:= l_rtti_context.GetType(p_c_object.ClassType);
        For l_c_rtti_property In l_c_rtti_type.GetProperties Do
        Begin
          l_property_name:= l_c_rtti_property.Name;
          l_property_type_name:= l_c_rtti_property.PropertyType.Name;
          l_type_kind_name:= f_display_type_kind(l_c_rtti_property.PropertyType.TypeKind);

          If l_c_rtti_property.PropertyType.TypeKind In k_basic_type_kinds
            Then l_property_value:=
                l_c_rtti_property.GetValue(p_c_object).ToString
            Else begin
                l_property_value:= '?' ;
                g_c_unknown_type_kind.Add(l_type_kind_name)
              end;

          // l_property_value:= l_property_value+ ' '+ l_c_rtti_property.ToString;

          l_display:= Format('%-19s %-16s %-15s %s',
              [l_property_name, l_property_type_name, l_type_kind_name, l_property_value]);
          if p_c_strings.IndexOf(l_display)< 0
            then p_c_strings.Add(l_display);
        End;
      End; // fill_property_list

    procedure fill_method_list(p_c_object: tObject; p_c_strings: tStrings);
      Var l_rtti_context: tRttiContext;
          l_c_rtti_type: tRttiType;
          l_c_rtti_method: tRttiMethod;
      begin
        p_c_strings.Clear;

        l_c_rtti_type:= l_rtti_context.GetType(p_c_object.ClassType);

        For l_c_rtti_method In l_c_rtti_type.GetMethods Do
          p_c_strings.Add(l_c_rtti_method.ToString);
      end; // fill_method_list

    function f_property_string_value(p_c_object: tObject; p_property_display: String): String;
      var l_index: integer;
          l_property_name: String;
          l_rtti_context: tRttiContext;
          l_c_rtti_type: tRttiType;
          l_c_rtti_property: tRttiProperty;
      begin
        Result:= '?';

        l_index:= 1; l_property_name:= '';
        while (l_index<= Length(p_property_display)) and (p_property_display[l_index]<> ' ')  do
        begin
          l_property_name:= l_property_name+ p_property_display[l_index];
          inc(l_index);
        end;

        l_c_rtti_type:= l_rtti_context.GetType(p_c_object.ClassType);
        For l_c_rtti_property In l_c_rtti_type.GetProperties Do
        Begin
          if SameText(l_property_name, l_c_rtti_property.Name)
            then begin
                if l_c_rtti_property.PropertyType.TypeKind= tkInteger
                  then begin
                      Result:= l_c_rtti_property.GetValue(p_c_object).ToString
                    end;
                Break;
              end;
        End;
      end; // f_property_string_value

    procedure set_property_value(p_c_object: tObject;
        p_property_display, p_string_value: String);
      var l_index: integer;
          l_property_name: String;
          l_rtti_context: tRttiContext;
          l_c_rtti_type: tRttiType;
          l_c_rtti_property: tRttiProperty;
      begin
        l_index:= 1; l_property_name:= '';
        while (l_index<= Length(p_property_display)) and (p_property_display[l_index]<> ' ')  do
        begin
          l_property_name:= l_property_name+ p_property_display[l_index];
          inc(l_index);
        end;

        l_c_rtti_type:= l_rtti_context.GetType(p_c_object.ClassType);
        For l_c_rtti_property In l_c_rtti_type.GetProperties Do
        Begin
          if SameText(l_property_name, l_c_rtti_property.Name)
            then begin
                display('found');
                if l_c_rtti_property.PropertyType.TypeKind= tkInteger
                  then begin
                      l_c_rtti_property.SetValue(p_c_object, StrToInt(p_string_value));
                    end;
                Break;
              end;
        End;
      end; // set_property_value

    procedure fill_ancestor_list(p_c_object: tObject; p_c_strings: tStrings);
      Var l_rtti_context: tRttiContext;
          l_c_rtti_type, l_c_ancestor_rtti_type: tRttiType;
      Begin
        p_c_strings.Clear;

        l_c_rtti_type:= l_rtti_context.GetType(p_c_object.ClassType);
(*
        repeat
          l_c_ancestor_rtti_type:= l_c_rtti_type.BaseType;
          if l_c_ancestor_rtti_type<> Nil
            then p_c_strings.Add(l_c_ancestor_rtti_type.Name);


          l_c_rtti_type:= l_c_ancestor_rtti_type;
        until l_c_ancestor_rtti_type= Nil;
*)
        l_c_ancestor_rtti_type:= l_c_rtti_type.BaseType;
        while l_c_ancestor_rtti_type<> Nil do
        begin
          p_c_strings.Insert(0, l_c_ancestor_rtti_type.Name);
          l_c_ancestor_rtti_type:= l_c_ancestor_rtti_type.BaseType;
        end; // while l_c_ancestor_rtti_type
      end; // fill_ancestor_list


end.
